import { Reducer, SyncStore } from './types';
/**
 * Creates a new synchronized state store with Redux-like patterns and React hooks integration.
 *
 * @example
 * ```typescript
 * // Define your state and action types
 * interface TodoState {
 *   items: string[];
 *   loading: boolean;
 * }
 *
 * type TodoAction =
 *   | { type: 'ADD_TODO'; payload: string }
 *   | { type: 'REMOVE_TODO'; payload: number }
 *   | { type: 'SET_LOADING'; payload: boolean };
 *
 * // Create your reducer
 * const todoReducer: Reducer<TodoState, TodoAction> = (state, action) => {
 *   switch (action.type) {
 *     case 'ADD_TODO':
 *       return { ...state, items: [...state.items, action.payload] };
 *     case 'REMOVE_TODO':
 *       return {
 *         ...state,
 *         items: state.items.filter((_, i) => i !== action.payload)
 *       };
 *     case 'SET_LOADING':
 *       return { ...state, loading: action.payload };
 *     default:
 *       return state;
 *   }
 * };
 *
 * // Create the store
 * const todoStore = createSyncStore(todoReducer, {
 *   items: [],
 *   loading: false
 * });
 *
 * // Use in components
 * function TodoList() {
 *   const [state, dispatch] = todoStore.useSyncReducer();
 *   const items = todoStore.useSyncSelector(s => s.items);
 *
 *   return (
 *     <ul>
 *       {items.map((item, index) => (
 *         <li key={index}>
 *           {item}
 *           <button onClick={() =>
 *             dispatch({ type: 'REMOVE_TODO', payload: index })
 *           }>
 *             Delete
 *           </button>
 *         </li>
 *       ))}
 *     </ul>
 *   );
 * }
 * ```
 *
 * @template S The type of the state managed by the store
 * @template A The type of the actions accepted by the reducer
 *
 * @param reducer A pure function that returns the next state
 * @param initialState The initial state value
 *
 * @returns A store instance with methods for state management and React hooks
 */
export declare function createSyncStore<S, A>(reducer: Reducer<S, A>, initialState: S): SyncStore<S, A>;
